"""
Auditor has one task: inspection of metadata.json and PPR.xml for required properties
"""
import json
import logging
import os.path
import re
import sys
from pathlib import Path
from typing import Dict, List

from bs4 import BeautifulSoup
from casa_envoy.interfaces import AuditorIF
from casa_envoy.schema import AbstractTextFile

orig_prettify = BeautifulSoup.prettify
r = re.compile(r"^(\s*)", re.MULTILINE)


def prettify_with_indent_width(self, encoding=None, formatter=None, indent_width=4):
    return r.sub(r"\1" * indent_width, orig_prettify(self, encoding, formatter))


BeautifulSoup.prettify = prettify_with_indent_width


def get_fields_for(filename: str) -> list:
    metadata_list = [
        "fileSetIds",
        "workflowName",
        "systemId",
        "productLocator",
        "destinationDirectory",
    ]
    ppr_list = ["RootDirectory", "RelativePath", "SdmIdentifier"]

    if ".xml" in filename:
        return ppr_list

    if ".json" in filename:
        return metadata_list


def get_parsed_xml_content(filename: str) -> BeautifulSoup:
    """
    Get parsed PPR content in the form of a BeautifulSoup object

    :param filename: Filename of PPR
    :return: Parsed contents
    """
    with open(filename) as file:
        return BeautifulSoup(file.read(), "xml")


def get_parsed_json_content(filename: str) -> Dict[str, str]:
    """
    Get parsed JSON metadata content in the form of a dict

    :param filename: Filename of PPR
    :return: Parsed contents
    """
    with open(filename) as file:
        return json.loads(file.read())


def get_value_for(filename: str, key: str) -> str:
    if ".xml" in filename:
        return get_parsed_xml_content(filename).find(key).string.strip()

    if ".json" in filename:
        return get_parsed_json_content(filename)[key]


def set_value_for(filename: str, key: str, value: str):
    """
    Set value for a given file, either XML or JSON
    NOTE: Does not keep formatting of original file

    :param filename: Filename of file to modify
    :param key: Key to modify
    :param value: New value to set
    """
    if ".xml" in filename:
        parsed_xml = get_parsed_xml_content(filename)
        parsed_xml.find(key).string = value
        with open(filename, "w") as file:
            file.write(parsed_xml.prettify(indent_width=4))

    if ".json" in filename:
        parsed_json = get_parsed_json_content(filename)
        parsed_json[key] = value
        with open(filename, "w") as file:
            file.write(json.dumps(parsed_json, indent=2))


class AuditFiles(AuditorIF):
    def __init__(self, files: List[str]):
        self.files = files
        self.logger = logging.getLogger("casa_envoy")
        self.logger.setLevel(logging.INFO)

    def read_file(self, filename: str) -> AbstractTextFile:
        if os.path.isfile(filename):
            with open(filename) as file:
                if ".json" in filename:
                    metadata = json.loads(file.read())
                    return AbstractTextFile(filename, json.dumps(metadata))
                else:
                    if ".xml" in filename:
                        ppr = file.read()
                        return AbstractTextFile(filename, ppr)

    def check_required_fields(self, file: AbstractTextFile, fields: list) -> bool:
        missing = []
        if ".xml" in file.filename:
            ppr_content = BeautifulSoup(file.content, "xml")
            for tag in fields:
                try:
                    ppr_content.find(tag).string
                except:
                    missing.append(tag)

        if ".json" in file.filename:
            for tag in fields:
                metadata = json.loads(file.content)
                if tag not in metadata or len(metadata[tag]) == 0:
                    missing.append(tag)
        if len(missing) > 0:
            print(f"Missing fields: {missing}")
            return False
        else:
            return True

    def audit(self) -> bool:
        invalid_files = []
        for file in self.files:
            f = self.read_file(file)
            valid = self.check_required_fields(file=f, fields=get_fields_for(f.filename))
            if not valid:
                invalid_files.append(f.filename)

        if len(invalid_files) != 0:
            self.logger.info(f"INVALID FILE FOUND: {invalid_files}")
            return False
        else:
            return True


class AuditDirectories(AuditorIF):
    def __init__(self, ppr: str):
        self.logger = logging.getLogger("casa_envoy")
        self.logger.setLevel(logging.INFO)
        self.rootDirectory = get_value_for(ppr, "RootDirectory")
        self.relative_path = get_value_for(ppr, "RelativePath")
        self.sdmId = get_value_for(ppr, "SdmIdentifier")

    def audit(self) -> bool:
        current = os.getcwd()
        needed = self.rootDirectory + "/" + self.relative_path
        if needed != current:
            self.logger.info("DIRECTORY ERROR: not in correct directory for processing.")
            return False
        else:
            working = Path(current + "/working").is_dir()
            products = Path(current + "/products").is_dir()
            rawdata = Path(current + "/rawdata").is_dir()

            if working and products and rawdata:
                self.logger.info("Checking for rawdata...")
                data = os.listdir(Path(current + "/rawdata/"))
                if len(data) > 0:
                    self.logger.info("Data is available. Proceeding...")
                    return True
                else:
                    self.logger.info("FAILURE: data not found in rawdata/")
                    return False
            else:
                self.logger.info(
                    "DIRECTORY ERROR: A directory is missing from the processing root directory."
                )
