"""
palaver definition: an unnecessarily elaborate or complex procedure
(A.K.A. CASA in a nutshell - fight me)
"""
import argparse
import glob
import json
import logging
import os
import pathlib
import re
import shutil
import subprocess
import sys
from contextlib import contextmanager, nullcontext
from typing import Dict

from casa_envoy.auditor import AuditDirectories, AuditFiles, set_value_for
from casa_envoy.interfaces import CasaLauncherIF
from casa_envoy.schema import AbstractTextFile
from pycapo import CapoConfig

"""
Launch CASA processing jobs via Workspaces
"""

logger = logging.getLogger("casa_envoy")
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler(sys.stdout))


class CalibrationLauncher(CasaLauncherIF):
    def __init__(self, ppr: AbstractTextFile, metadata: AbstractTextFile):
        self.ppr = ppr
        self.metadata = metadata

    def run(self):
        logger.info("RUNNING CASA!")
        shutil.copy("PPR.xml", "./working")
        os.chdir("./working")
        result = subprocess.Popen(
            "PYTHONPATH='' xvfb-run -e ${PWD}/xvfb-run.err.txt -d -s \"-screen 0 800x600x16\" "
            "${CASA_HOME}/bin/casa --pipeline --nogui --nologger -c "
            "${CASA_HOME}/pipeline/pipeline/runvlapipeline.py ${PPR_FILENAME} || true",
            shell=True,
            executable="/bin/bash",
            stdout=sys.stdout,
            stderr=sys.stderr,
        )
        return result.communicate()

    def setup_environment(self, parameters: dict):
        os.environ["SCIPIPE_ROOTDIR"] = parameters["rootDirectory"]
        os.environ["CASA_HOME"] = parameters["homeForReprocessing"]
        os.environ["PPR_FILENAME"] = str(self.ppr)

        logger.info("Checking processing environment:")
        env1 = os.environ.get("SCIPIPE_ROOTDIR")
        logger.info(f"SCIPIPE_ROOTDIR: {env1}")
        env2 = os.environ.get("CASA_HOME")
        logger.info(f"CASA_HOME: {env2}")
        env3 = os.environ.get("PPR_FILENAME")
        logger.info(f"PPR_FILENAME: {env3}")
        if "None" in [env1, env2, env3]:
            logger.info("Environment setup Failed!")
            sys.exit(1)
        else:
            logger.info("Environment ready for processing")

    def check_logs(self, parent_path: str):
        logger.info("CHECKING CASA LOGS!")
        # make sure we are in the correct directory to find log file
        if not os.getcwd().endswith("/working"):
            os.chdir(parent_path + "/working")

        casa_logs = glob.glob("casa-*.log")

        for file in casa_logs:
            if re.match("^.*SEVERE\sflagmanager.*$", open(file).read()):
                logger.info("CASA ERROR!")
            else:
                logger.info("CASA Success!")


@contextmanager
def substitute_ppr_paths_for_condor(ppr: str, settings: Dict[str, str]):
    """
    Context manager that upon entry creates a copy of the existing PPR with original path values intact, then
    modifies the original PPR with the correct path to the HTCondor working directory; upon exit, the function
    deletes the condor-modified PPR and renames the copy with intact path values to PPR.xml

    :param ppr File name of PPR
    :param settings Dictionary of casa_envoy environment settings
    """
    cwd = pathlib.Path(os.getcwd())
    original_rootdir = settings["rootDirectory"]
    this_dir = str(cwd.stem)
    up_one_dir = str(cwd.parent)
    original_ppr = "PPR-original.xml"

    try:
        # Copy PPR
        shutil.copyfile(ppr, original_ppr)
        # Modify SCIPIPE_ROOTDIR
        settings["rootDirectory"] = up_one_dir
        # Modify PPR.xml path values with . and ..
        set_value_for(ppr, "RootDirectory", up_one_dir)
        set_value_for(ppr, "RelativePath", this_dir)
        yield
    finally:
        # Reassign SCIPIPE_ROOTDIR to original value
        settings["rootDirectory"] = original_rootdir
        # Remove modifed PPR.xml
        os.remove(ppr)
        # Rename PPR with original values back to PPR.xml
        os.rename(original_ppr, "PPR.xml")


def _get_settings():
    use_casa = CapoConfig().getboolean("edu.nrao.archive.workspaces.ProcessingSettings.useCasa")
    use_condor = CapoConfig().getboolean("edu.nrao.archive.workspaces.ProcessingSettings.useCondor")
    root_dir = CapoConfig().settings("edu.nrao.archive.workspaces.ProcessingSettings").rootDirectory
    casa_home = (
        CapoConfig().settings("edu.nrao.archive.workflow.config.CasaVersions").homeForReprocessing
    )

    return {
        "useCasa": use_casa,
        "useCondor": use_condor,
        "homeForReprocessing": casa_home,
        "rootDirectory": root_dir,
    }


def parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        description="Workspaces CASA processing launcher",
        formatter_class=argparse.RawTextHelpFormatter,
    )
    parser.add_argument(
        "--standard-cal",
        nargs=2,
        action="store",
        required=False,
        help="run the standard calibration CASA pipeline",
    )
    return parser


def check_calibratable(metadata_filename) -> bool:
    with open(str(metadata_filename)) as json_meta:
        metadata = json.loads(json_meta.read())
        spl = metadata["productLocator"]
        if "execblock" in spl:
            return True
        else:
            logger.info("SPL ERROR: This product locator is not calibratable!")
            return False


def run_audit(ppr: str, metadata: str):
    dir_audit = AuditDirectories(ppr).audit()
    if dir_audit:
        logger.info("Directory audit successful!")
    else:
        logger.info("FAILURE: directory structure audit was unsuccessful!")
        sys.exit(1)

    audit = AuditFiles([ppr, metadata]).audit()
    if audit:
        logger.info("File audit successful!")
    else:
        logger.info("FAILURE: file audit was unsuccessful!")
        sys.exit(1)


def main():
    args = parser().parse_args()
    metadata = args.standard_cal[0]
    ppr = args.standard_cal[1]
    settings = _get_settings()

    with substitute_ppr_paths_for_condor(ppr, settings) if settings["useCondor"] else nullcontext():
        # Context manager that performs PPR working path substitution if HTCondor is used
        # nullcontext() is simply a way of doing a no-op if Condor is not used
        run_audit(ppr, metadata)

        if check_calibratable(metadata):
            path = os.getcwd()
            if args.standard_cal:
                if settings.get("useCasa"):
                    launcher = CalibrationLauncher(ppr, metadata)
                    launcher.setup_environment(settings)
                    launcher.run()
                    launcher.check_logs(parent_path=path)
                    # make sure we return to the parent directory after processing
                    os.chdir(path)
                else:
                    logger.info("RUNNING VELA!")
                    subprocess.Popen(["vela", "--standard-cal", metadata, ppr])
        else:
            logger.info("TYPE ERROR: Provided SPL is not type execution block!")
            sys.exit(1)
